/*-
|> File
|| ps_types.h
||
|> Description
||
||
|> Comments
||
||
|> History
|| 95061 _mf Created.
||
||==================================================================|>
*/
#ifndef _dir_ps_types_h
#define _dir_ps_types_h

#ifndef _NOIDENT
#ident "$Id: ps_types.h,v 2.1 1995/10/04 19:21:30 mreynold Exp $"
#endif


#define PSEUDOCOLOR_VISUAL 0x20
#define TRUECOLOR_VISUAL 0x21
/*
   the following structure remembers for each file descriptor its
   state.  In particular, we need to know if it is busy or free
   and if it is in use, by whom.
 */

typedef int             FD;

struct FDDescriptor
{
  Bool                    Busy;
  int                     (*InputHandler) ();
};

typedef struct replyDel
{
  unsigned char          *reply;  /* if there's reply to substitute */
  int                     replyLength;
}
replyDelRec            , *replyDelPtr;

/* this is the structure for adding in printer options. I started off with
   a flag for indexed colorspaces, because our stupid versatec doesn't
   support them... */

typedef struct printOptions
{                               /* options used in the program. */
  int                     indexCspaceSupport;
}
printOptionsRec        , *printOptionsPtr;

struct PrintClient
{
  Window                  win;
  Bool                    serverWin;  /* X server window or ours */
  char                   *printfile;
  char                   *correctFile;  /* file of printing color corrections */
  int                     numCorrect;   /* length of color correct vector */
  float                 **corrects;   /* correction vectors */
  FILE                   *printfp;
  unsigned char          *buffer;
  int                     bufsize;
  int                     (*bytefn) ();
  int                     sequence;
  int                     preserveAspect;   /* flag for aspect ratio */
  int                     fontScale;  /* font scale factor */
  int                     pixmapScale;  /* pixmap scale factor */
  int                     pageWidth;  /* page size in points */
  int                     pageHeight;
  int                     printDirect;  /* to file or direct to printer */
  int                     landscape;  /* 0=portrait, 1=landscape */
  char			  *prologFile;	/* for postscript prologs */
};

/* window information */

typedef struct winfo
{
  Window                  parent;
  Window                  win;
  XWindowAttributes       attr;
  int                     created;
  int                     fd;
}
winfoRec               , *winfoPtr;

/* window table */

typedef struct winfoTable
{
  winfoPtr                winTable;
  int                     numWin;
}
winfoTableRec          , *winfoTablePtr;

/* colormap information */

typedef struct cmapinfo
{
  XColor                  colors[NUM_COLORS];
  int                     numAlloc[NUM_COLORS];
  int                     numColors;
  int                     writeable[NUM_COLORS];
  Colormap                cmap;
  int			  visualID;
  int                     fd;
}
cmapinfoRec            , *cmapinfoPtr;

/* table of colormaps */

typedef struct cmapinfoTable
{
  cmapinfoPtr             cmapTable;
  int                     numCmap;
}
cmapinfoTableRec       , *cmapinfoTablePtr;

/* GC information */

typedef struct gcinfo
{
  XID                     gcid;
  XGCValues               attr;
  XRectangle             *clipRects;
  int                     numClipRects;
  char                   *dashes;
  int                     numDashes;
  int                     fd;
}
gcinfoRec              , *gcinfoPtr;

/* gc table */

typedef struct gcinfoTable
{
  gcinfoPtr               gcTable;
  int                     numGc;
}
gcinfoTableRec         , *gcinfoTablePtr;

/* for enum-string lookup tables */

typedef struct lookup
{
  int                     tag;
  char                   *name;
}
lookupRec              , *lookupPtr;

typedef struct font
{
  FD                      fd;
  int                     nfid;
  Font                    fid[MAXFID];
  Font                    serverFid;
  char                   *xfontName;
  char                   *postName;
  XFontStruct            *fontInfo;
}
fontRec                , *fontPtr;

typedef struct fontTable
{
  fontPtr                 fontTable;
  int                     numFonts;
}
fontTableRec           , *fontTablePtr;

typedef struct atom
{
  char                   *name;
  int                     id;
}
atomRec                , *atomPtr;

typedef struct atomTable
{
  atomPtr                 atomTable;
  int                     numAtoms;
}
atomTableRec           , *atomTablePtr;

#endif

/*- end ps_types.h
*/
